
/**
 * @file   gentable.c
 * @author David Bremner
 * @date   Thu Apr 11 15:37:35 2002
 * 
 * @brief  Generate the 3 term Grassman Pluecker relations
 * 
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "parser.h"
#include "gparams.h"
#include "combinat.h"
#include "oms_error.h"
#include "facet_list.h"
#include "signed_index.h"
#include "basis_equation.h"
#include "eqout.h"
#include "path.h"
static   be_block_p pluecker,conv_eq;
   
static void usage(char ** argv){
  fprintf(stderr,"usage: %s: [options] -r rank -n elements\n",argv[0]);
  fprintf(stderr,"-v \texpand indices\n");
  fprintf(stderr,"-e \tuse external indices\n");

}


/** 
 * Generate coefficents for one row of pluecker table
 * 
 * @param pluecker 
 * @param row 
 */

static void pluecker_ieq(int n, 
			 int r,
			 be_block_p pluecker, 
			 int row, int external, 
			 int verbose, int *buf){
  int i,j;

  
  for (j=0; j<be_cols(pluecker); j++){    
    printf(" %c",be_sign(pluecker,row,j) <0 ? '-' : ' ');
    if (verbose){
      output_bindex(n,r,be_bindex(pluecker,row,j),buf);
    } else {
      printf("%02d", be_bindex(pluecker,row,j)+external);
    }
  }
  printf("\n");
}



int main(int argc, char** argv){
  int c;
  int n=5,r=3;
  int num_bases,num_eqns;
  int external=0;
  int verbose=0;
  int *buf=NULL;

  int row,i;
  sarray_p found;

  int eqn_count;
  
  params_p params=malloc(sizeof(params_t));

  while((c=getopt(argc,argv,"o:n:r:ev"))!=-1){
    switch(c){
    case 'o':
      freopen(optarg,"w",stdout);
      assert(stdout);
      break;
    case 'n':
      n=atoi(optarg);
      break;
    case 'r':
      r=atoi(optarg);
      break;
    case 'e':
      external=1;
      break;
    case 'v':
      verbose=1;
      break;
    default:
      usage(argv);
      exit(1);

    }
  }

  num_eqns=binomial(n,r+2)*binomial(r+2,4); 
  num_bases=binomial(n,r);

  if (verbose)
    buf=calloc(r+1,sizeof(int));

  pluecker=be_alloc(num_eqns,6,num_bases);
  pl_init(pluecker,n,r);


  for (row=0; row< num_eqns; row++){
    pluecker_ieq(n,r,pluecker,row,external,verbose,buf);
  }

  if (verbose)
    free(buf);
  return(0);
}
