/**
 * @file   signed_set.c
 * @author David Bremner
 * @date   Wed Jul 10 16:02:22 2002
 * 
 * @brief  keep a set of signed basis indices.
 * 
 * 
 */

#include "nalloc.h"
#include "nu_sign.h"
#include "nu_sign_set.h"
#include "nu_signed_index.h"
#include "nss_slots.h"
#include "error_message.h"
#include "nu_int_to_symbol.h"

TAG_DECLARE(nss_fixup);

nu_sign_set_p nss_new(natural_t ssize){
  nu_sign_set_p new;
  int i;

  new=malloc(sizeof(nu_sign_set_t));
  new->size=ssize;
  new->present=calloc(sizeof(bool),ssize);
  new->sign=calloc(sizeof(nu_sign_t),ssize);
  for (i=0; i<ssize; i++){
    new->sign[i]=SIGN_ANY;
  }
    
  new->queue=iq_new(ssize);
  return new;
}
void nss_dump(nu_sign_set_p ss){
  int i;
  for (i=0; i<iq_count(ss->queue); i++){
    nu_signed_index_t si=nss_next(ss);
    fprintf(stderr,"%d:%c ",nsi_index(si),sign2char(nsi_sign(si)));
  }
}
void nss_fixup(na_node_p node, slot_t slot){
  nu_sign_set_p ptr;

  ptr=na_access(node,slot);
  ptr->present=na_access(node,slot+nss_OFF_present);
  ptr->sign=na_access(node,slot+nss_OFF_sign);
  ptr->queue=na_access(node,slot+nss_OFF_queue);
}
na_node_p nss_nalloc(na_node_p node, slot_t slot,natural_t ssize){
  nu_sign_set_p new;
  int i;

  
  node=na_alloc(node,slot,sizeof(nu_sign_set_t));
  node=na_alloc(node,slot+nss_OFF_present,sizeof(bool)*ssize);
  node=na_alloc(node,slot+nss_OFF_sign,sizeof(nu_sign_t)*ssize);
  node=iq_nalloc(node,slot+nss_OFF_queue,ssize);

  na_set_fixup(node,slot,TAG(nss_fixup));
  nss_fixup(node,slot);
  
  new=na_access(node,slot);
  new->size=ssize;
  for (i=0; i<ssize; i++){
    new->sign[i]=SIGN_ANY;
    new->present[i]=0;
  }
  return node;
}

void nss_destroy(nu_sign_set_p S){
  iq_destroy(S->queue);
  free(S->sign);
  free(S->present);
  free(S);
}

bool nss_add(nu_sign_set_p S, int k, nu_sign_t sign){
  
  err_assert(k<S->size && k>=0);
  
  if (S->present[k]){
    if (sign_subset(S->sign[k],sign))
      return TRUE;

    if (!sign_compatible(sign,S->sign[k]))
      return FALSE;


    S->sign[k]=sign_intersection(sign,S->sign[k]);
  } else {
    S->present[k]=TRUE;
    S->sign[k]=sign;
    iq_enq(S->queue,k);
  }
  
  return(TRUE);

}

int nss_empty(nu_sign_set_p S){
  return iq_empty(S->queue);
}

int nss_count(nu_sign_set_p S){
  return iq_count(S->queue);
}

nu_signed_index_t  nss_deq(nu_sign_set_p S){
  int k;

  k=iq_deq(S->queue);

  S->present[k]=FALSE;
  return nsi_encode(k,S->sign[k]);

}




array_index_t  nss_deq_index(nu_sign_set_p S){
  int k;

  k=iq_deq(S->queue);
  
  S->present[k]=0;
  return k;
}


nu_signed_index_t  nss_next(nu_sign_set_p S){
  int k;

  k=iq_deq(S->queue);

  iq_enq(S->queue,k);
  return nsi_encode(k,S->sign[k]);
  
  

}
