/**
 * @file   signed_set.c
 * @author David Bremner
 * @date   Wed Jul 10 16:02:22 2002
 * 
 * @brief  data structure for representing signed sets, i.e. 
 *		an element may be not present, positively present, or 
 *  negatively present.
 * 
 * 
 */
#include "signed_set.h"
#include "error_message.h"

signed_set_p ss_new(natural_t ssize){
  signed_set_p new;

  new=calloc(sizeof(signed_set_t)+ssize*sizeof(int),1);
  new->size=ssize;
  new->queue=iq_new(ssize);
  return new;
}

void ss_destroy(signed_set_p S){
  iq_destroy(S->queue);
  free(S);
}

bool ss_add(signed_set_p S, int k, int sign){
  
  err_assert(k<S->size && k>=0);
  
  if (S->present[k]){
    if (S->present[k]!=sign){
      return(FALSE);
    } else {
      return(TRUE);
    }
  } else {
    S->present[k]=sign;
    iq_enq(S->queue,k);
    return(TRUE);
  }
  
}

bool ss_empty(signed_set_p S){
  return iq_empty(S->queue);
}

int ss_count(signed_set_p S){
  return iq_count(S->queue);
}

int ss_deq(signed_set_p S){
  int k,rval;

  k=iq_deq(S->queue);
  
  rval=si_encode(k,S->present[k]);
  S->present[k]=0;
  return rval;
}

